#include <cmath>
using namespace std;
#include <cstdlib>
using namespace std;
#include <cstdio>
using namespace std;
#include <fstream>
using namespace std;
#include <iostream>
using namespace std;

class GFNS
{
  //*****************************************************************
  //Numerical Implementations of Various Gamma Functions
  //*****************************************************************
  /*
    digamma = low order approximation of Eq. 6.3.18 in Abrahomwitz
    trigamma = low order approximation of 6.4.11
    gammalog = approximation of log gamma function taken from Numerical Recipies
    polygamma = higher order approximation of digamma and trigamma
    e.g. ploygamma(3.0,1) = trigamma function at x = 3.0
    polygamma(3.0,0) = digamma function at x = 3.0

    polygamma is much closer to matlab psi function for large x

    gamrnd(a,r) = gives a gamma random derivate with parameter a and random object r
    gamrnd_int(a,r) is used by gamrnd and gives a random deviate with integer a 
  */

public:
  double fSmallNum;
	
  //loggamma coefficients
  double *cof;

  //Bernoulli constants for polygamma function
  double *bernoulli;

  double digamma(double x);
  double digamms(double x);
  double trigamma(double x);
  double polygamma(double x, int order);
  double trigamms(double x);
  double gamrnd(double A);
  double gamrnd_int(int a);
  double gammalog(double x);
  //Constructor
  GFNS();
  ~GFNS();
};//class

GFNS::~GFNS()
{
  delete [] cof;
  delete [] bernoulli;
}

GFNS::GFNS()
{
  fSmallNum = 0.0;

  //loggamma coefficients
  cof = new double[6];
  cof[0] = 76.18009172947146;
  cof[1] = -86.50532032941677;
  cof[2] = 24.01409824083091;
  cof[3] = -1.231739572450155;
  cof[4] = 0.1208650973866179e-2;
  cof[5] = -0.5395239384953e-5;
  
  

  //Bernoulli constants for polygamma function
  bernoulli = new double[20];
  bernoulli[0] = 1.66666666666666667e-01;
  bernoulli[1] = -3.33333333333333333e-02;
  bernoulli[2] = 2.38095238095238095e-02;
  bernoulli[3] = -3.33333333333333333e-02;
  bernoulli[4] = 7.57575757575757576e-02;
  bernoulli[5] = -2.53113553113553114e-01;
  bernoulli[6] = 1.16666666666666667e+00;
  bernoulli[7] = -7.09215686274509804e+00;
  bernoulli[8] = 5.49711779448621554e+01;
  bernoulli[9] = -5.29124242424242424e+02;
  bernoulli[10] = 6.19212318840579710e+03;
  bernoulli[11] = -8.65802531135531136e+04;
  bernoulli[12] = 1.42551716666666667e+06;
  bernoulli[13] = -2.72982310678160920e+07;
  bernoulli[14] = 6.01580873900642368e+08;
  bernoulli[15] = -1.51163157670921569e+10;
  bernoulli[16] = 4.29614643061166667e+11;
  bernoulli[17] = -1.37116552050883328e+13;
  bernoulli[18] = 4.88332318973593167e+14;
  bernoulli[19] = -1.92965793419400681e+15;
}

double GFNS::polygamma(double x, int order)
{
  double m_base = 2.0;
  double m_digits = 5.0;  
  double m_exp  = 100.0;
  double m_LN10 = 10.0;
  double psum   = 0.0;
  double toler1,toler2,toler3,toler4,temp,scale,xx,ad,square,temp2,temp3,sum;
  int i;

  toler1=pow(m_base,-(m_digits-1)) * 0.5;
  if (toler1 < 0.5e-18) toler1 = 0.5e-18;
  toler2=m_exp * log(fSmallNum + m_base) - 3 * m_LN10;

  temp=m_digits * log10(m_base) - 3;
  toler3 = 4.5 + 0.4 * temp + order * ((0.00868 + temp * 0.00604) * temp + 0.21);

  scale = 1.0;

  if (order==0) scale = -1;
  else {for (i=2; i<=order; i++) scale*=-i;}

  double logx=log(fSmallNum + x);

  if (( logx * (order + 1)) > toler2) {psum=0.0;}
  else if (x < toler1) {psum = scale * pow(x,(double) -(order + 1));}

  if (x < toler3)
    {
      ad = floor(1 + toler3 - x);
      xx = x + ad;
    }
  else
    {
      xx = x;
      ad = 0.0;
    }

  //Bernoulli approximation
  toler4 = 0.5 * toler1/xx;

  square = (1/(xx * xx));
  temp = (order + 1) * square * 0.5;
  sum = 0.5/xx + temp * bernoulli[0];
  temp2=2.0;
  for (i=1;i<20;i++)
    {
      temp *= square * ((temp2+order+1.0)/(temp2+1.0))*((temp2+order)/(temp2+2.0));
      temp3 = temp * bernoulli[i];
      sum += temp3;
      if (fabs(temp3) < toler4) break;
      temp2+=2.0;
    }
  if (order==0)
    {
      for (i=(int)ad-1 ; i>=0 ; i--) sum += 1.0/(x + i);
      sum = log(fSmallNum + xx) - sum;
    }
  else
    {
      sum = (sum + 1.0/order) * pow(xx,(double)-order);
      for (i=(int)ad-1; i>=0 ; i--) sum += pow(x+i, -(order+1.0));
      sum *= scale;
    }
  psum=sum;
  return psum;
}

double GFNS::digamma(double x)
{
  double a,b,g,cor,z;
  a=1e-8;
  b=10.0;
  g=-0.57721566490153;
  cor=0.0;

  if(x<0.0)
    {
      cout << "Digamma argument less than zero" << endl;
      return -1;
    }
  else if(x<a)
    return g - 1.0/x;
  else if(x>b)
    return digamms(x);
  else
    {
      z=x;
      do{
	cor=cor+1.0/z++;
      }while(z<b);
      return digamms(z)-cor;
    }
}

double GFNS::digamms(double x)
{
  double y,s3=1.0/12,s4=1.0/120,s5=1.0/252;
  y=1.0/(x*x);
  return log(fSmallNum + x)-(0.5/x)-y*(s3-y*(s4-y*s5));
}
double GFNS::trigamma(double x)
{
  double a=1.0e-8,b=12.0,cor=0.0,z;
  if(x<0.0)
    {
      cout << "Digamma argument less than zero" << endl;
      return -1;
    }
  else if(x<a)
    return 1.0/(x*x);
  else if(x>b)
    return trigamms(x);
  else
    {
      z=x;
      do{
	cor=cor+1.0/(z*z);
	z++;
      }while(z<b);
      return trigamms(z)+cor;
    }

}

double GFNS::trigamms(double x)
{
  double y,b2=1.0/6,b4=-1.0/30,b6=1.0/42;
  y=1.0/(x*x);
  return 0.5*y+(1.0+y*(b2+y*(b4 + y*(b6+y*b4))))/x;
}

double GFNS::gamrnd(double A)
{
  if (A <= 5)
    {
      double b,p,inva,res=0;
      double EXP= 2.71828182;
      double a,f;
      int g,rub=1;

      f= floor( A);
      a= A-f;
      g= (int) f;

      if( a!=0)
	{
	  inva= 1/a;
	  b= 1.0 + a/EXP;

	  while(rub==1)
	    {
	      p= b*((double)rand()/(double)RAND_MAX);
	      if( p<=1.0 )
		{
		  res= pow(p, inva);
		  if( ((double)rand()/(double)RAND_MAX) <= exp( -res)) break;
		}
	      else
		{
		  res= - log( (b-p)/a);
		  if( ((double)rand()/(double)RAND_MAX) <= 1.0/pow(res, 1-a)) break;
		}
	    }
	}

      if ( g>0)
	res += gamrnd_int(g);

      return(res );
    }
  else /*Use the same function as integer*/
    {
      double x,y,v,sqrt2a1,asub1,y2,ans=0.0;
      int rub=1;
      sqrt2a1 = sqrt (2*A - 1);
      asub1 = A-1;
      while (rub==1)
	{
	  y = tan (3.1415926536*((double)rand()/(double)RAND_MAX));
	  y2 = y*sqrt2a1;
	  if ((x = y2 + asub1) <= 0)
	    continue;
	  v = ((double)rand()/(double)RAND_MAX);
	  if (v <= (1+y*y)*exp((asub1)*log(x/asub1)-y2))
	    {
	      ans = x;
	      break;
	    }
	}
      return ans;
    }
}

double GFNS::gamrnd_int(int a)
{
  double GAMMA_TURNING_POINT = 7.0;
  //Generate random from gamma with integer a
  if (a>GAMMA_TURNING_POINT)
    {
      int rub=1;
      double x,y,v,sqrt2a1,asub1,y2;
      double ans = 0.0;
      sqrt2a1 = sqrt ((double)(2*a - 1));
      asub1 = a-1;
      while (rub==1)
	{
	  y = tan (3.1415926536*((double)rand()/(double)RAND_MAX));
	  y2 = y*sqrt2a1;
	  x = y2 + asub1;
	  if (x <= 0)
	    continue;
	  v = ((double)rand()/(double)RAND_MAX);
	  if (v <= (1+y*y)*exp((asub1)*log(x/asub1)-y2))
	    {
	      ans =  x;
	      break;
	    }
	}
      return ans;
    }
  else
    {
      double ans=1;
      while (a-- > 0)
	ans *= ((double)rand()/(double)RAND_MAX);
      return (-log(ans));
    }
}
 
double GFNS::gammalog(double x)
{
  int k;
  double w, t, y, v;
  double a[] = {
    9.967270908702825e-5, -1.9831672170162227e-4,
    -0.00117085315349625822, 0.00722012810948319552,
    -0.0096221300936780297, -0.04219772092994235254,
    0.16653861065243609743, -0.04200263501129018037,
    -0.65587807152061930091, 0.57721566490153514421,
    0.99999999999999999764,
    4.67209725901142e-5, -6.812300803992063e-5,
    -0.00132531159076610073, 0.0073352117810720277,
    -0.00968095666383935949, -0.0421764281187354028,
    0.16653313644244428256, -0.04200165481709274859,
    -0.65587818792782740945, 0.57721567315209190522,
    0.99999999973565236061
  };
  double b[] = {
    -4.587497028e-11, 1.902363396e-10,
    8.6377323367e-10, 1.15513678861e-8,
    -2.556403058605e-8, -1.5236723372486e-7,
    -3.1680510638574e-6, 1.22903704923381e-6,
    2.334372474572637e-5, 0.00111544038088797696,
    0.00344717051723468982, 0.03198287045148788384,
    -0.32705333652955399526, 0.40120442440953927615,
    -5.184290387e-11, -8.3355121068e-10,
    -2.56167239813e-9, 1.455875381397e-8,
    1.3512178394703e-7, 2.9898826810905e-7,
    -3.58107254612779e-6, -2.445260816156224e-5,
    -4.417127762011821e-5, 0.00112859455189416567,
    0.00804694454346728197, 0.04919775747126691372,
    -0.24818372840948854178, 0.11071780856646862561,
    3.0279161576e-10, 1.60742167357e-9,
    -4.05596009522e-9, -5.089259920266e-8,
    -2.029496209743e-8, 1.35130272477793e-6,
    3.91430041115376e-6, -2.871505678061895e-5,
    -2.3052137536922035e-4, 4.5534656385400747e-4,
    0.01153444585593040046, 0.07924014651650476036,
    -0.12152192626936502982, -0.07916438300260539592,
    -5.091914958e-10, -1.15274986907e-9,
    1.237873512188e-8, 2.937383549209e-8,
    -3.0621450667958e-7, -7.7409414949954e-7,
    8.16753874325579e-6, 2.412433382517375e-5,
    -2.60612176060637e-4, -9.1000087658659231e-4,
    0.01068093850598380797, 0.11395654404408482305,
    0.07209569059984075595, -0.10971041451764266684,
    4.0119897187e-10, -1.3224526679e-10,
    -1.002723190355e-8, 2.569249716518e-8,
    2.0336011868466e-7, -1.1809768272606e-6,
    -3.00660303810663e-6, 4.402212897757763e-5,
    -1.462405876235375e-5, -0.0016487379559600128,
    0.00513927520866443706, 0.13843580753590579416,
    0.32730190978254056722, 0.08588339725978624973,
    -1.5413428348e-10, 6.4905779353e-10,
    1.60702811151e-9, -2.655645793815e-8,
    7.619544277956e-8, 4.7604380765353e-7,
    -4.90748870866195e-6, 8.21513040821212e-6,
    1.4804944070262948e-4, -0.00122152255762163238,
    -8.7425289205498532e-4, 0.1443870369965796831,
    0.61315889733595543766, 0.55513708159976477557,
    1.049740243e-11, -2.5832017855e-10,
    1.39591845075e-9, -2.1177278325e-10,
    -5.082950464905e-8, 3.7801785193343e-7,
    -7.3982266659145e-7, -1.088918441519888e-5,
    1.2491810452478905e-4, -4.9171790705139895e-4,
    -0.0042570708944826646, 0.13595080378472757216,
    0.89518356003149514744, 1.31073912535196238583
  };
  double c[] = {
    1.16333640008e-8, -8.33156123568e-8,
    3.832869977018e-7, -1.5814047847688e-6,
    6.50106723241e-6, -2.74514060128677e-5,
    1.209015360925566e-4, -5.666333178228163e-4,
    0.0029294103665559733, -0.0180340086069185819,
    0.1651788780501166204, 1.1031566406452431944,
    1.2009736023470742248,
    1.3842760642e-9, -6.9417501176e-9,
    3.42976459827e-8, -1.785317236779e-7,
    9.525947257118e-7, -5.2483007560905e-6,
    3.02364659535708e-5, -1.858396115473822e-4,
    0.0012634378559425382, -0.0102594702201954322,
    0.1243625515195050218, 1.3888709263595291174,
    2.4537365708424422209,
    1.298977078e-10, -8.02957489e-10,
    4.945484615e-9, -3.17563534834e-8,
    2.092136698089e-7, -1.4252023958462e-6,
    1.01652510114008e-5, -7.74550502862323e-5,
    6.537746948291078e-4, -0.006601491253552183,
    0.0996711934948138193, 1.6110931485817511402,
    3.9578139676187162939,
    1.83995642e-11, -1.353537034e-10,
    9.984676809e-10, -7.6346363974e-9,
    5.99311464148e-8, -4.868554120177e-7,
    4.1441957716669e-6, -3.77160856623282e-5,
    3.805693126824884e-4, -0.0045979851178130194,
    0.0831422678749791178, 1.7929113303999329439,
    5.6625620598571415285,
    3.4858778e-12, -2.97587783e-11,
    2.557677575e-10, -2.2705728282e-9,
    2.0702499245e-8, -1.954426390917e-7,
    1.9343161886722e-6, -2.0479024910257e-5,
    2.405181940241215e-4, -0.0033842087561074799,
    0.0713079483483518997, 1.9467574842460867884,
    7.5343642367587329552
  };
  double d[] = {
    -0.00163312359200500807, 8.3644533703385956e-4,
    -5.9518947575728181e-4, 7.9365057505415415e-4,
    -0.00277777777735463043, 0.08333333333333309869,
    0.91893853320467274178
  };

  w = x;
  if (x < 0) {
    w = 1 - x;
  }
  if (w < 0.5) {
    k = w < 0.25 ? 0 : 11;
    y = ((((((((((a[k] * w + a[k + 1]) * w +
		 a[k + 2]) * w + a[k + 3]) * w + a[k + 4]) * w +
	      a[k + 5]) * w + a[k + 6]) * w + a[k + 7]) * w +
	   a[k + 8]) * w + a[k + 9]) * w + a[k + 10]) * w;
    y = -log(y);
  } else if (w < 3.5) {
    t = w - 4.5 / (w + 0.5);
    k = (int) (t + 4);
    t -= k - 3.5;
    k *= 14;
    y = ((((((((((((b[k] * t + b[k + 1]) * t +
		   b[k + 2]) * t + b[k + 3]) * t + b[k + 4]) * t +
		b[k + 5]) * t + b[k + 6]) * t + b[k + 7]) * t +
	     b[k + 8]) * t + b[k + 9]) * t + b[k + 10]) * t +
	  b[k + 11]) * t + b[k + 12]) * t + b[k + 13];
  } else if (w < 8) {
    k = ((int) w) - 3;
    t = w - (k + 3.5);
    k *= 13;
    y = (((((((((((c[k] * t + c[k + 1]) * t +
		  c[k + 2]) * t + c[k + 3]) * t + c[k + 4]) * t +
	       c[k + 5]) * t + c[k + 6]) * t + c[k + 7]) * t +
	    c[k + 8]) * t + c[k + 9]) * t + c[k + 10]) * t +
	 c[k + 11]) * t + c[k + 12];
  } else {
    v = 1 / w;
    t = v * v;
    y = (((((d[0] * t + d[1]) * t + d[2]) * t +
	   d[3]) * t + d[4]) * t + d[5]) * v + d[6];
    y += (w - 0.5) * log(w) - w;
  }
  if (x < 0) {
    y = log((3.141592653589793238 / sin(x * 3.141592653589793238))) - y;
  }
  return y;
}
